@@ -4,12 +4,9 @@
 ## 
 ## params = the initial particles for the parameter values
 ##          these are drawn from the prior distribution for the parameters
-##          if a parameter is being held fixed, it is given as a row of NA's
 ## est = names of parameters to estimate.  Other parameters are not updated.
-## Np = number of particles
-## discount = delta, introduced in section 3.3 in Liu & West
+## smooth = parameter 'h' from AGM
 ## ntries = number of samplesto draw from x_t+1|x(k)_t to estimate mean of mu(k)_t+1 as in sect 2.2 Liu & West
-## ntimes = number of timesteps in observation vector
 ## lower  = lower bounds on prior
 ## upper  = upper bounds on prior
 
@@ -18,7 +15,7 @@ setGeneric("bsmc",function(object,...)standardGeneric("bsmc"))
 setMethod(
           "bsmc",
           "pomp",
-          function (object, params, est,
+          function (object, params, Np, est,
                     smooth = 0.1,
                     ntries = 1,
                     tol = 1e-17,
@@ -30,7 +27,8 @@ setMethod(
 
             if (missing(seed)) seed <- NULL
             if (!is.null(seed)) {
-              if (!exists(".Random.seed",where=.GlobalEnv)) { # need to initialize the RNG
+              if (!exists(".Random.seed",where=.GlobalEnv)) {
+                ## need to initialize the RNG
                 runif(1)
               }
               save.seed <- get(".Random.seed",pos=.GlobalEnv)
@@ -47,7 +45,10 @@ setMethod(
               }
             }
 
-            Np <- NCOL(params)
+            if (missing(Np)) Np <- NCOL(params)
+            else if (is.matrix(params)&&(Np!=ncol(params)))
+              stop(error.prefix,sQuote("Np")," cannot be other than ",sQuote("ncol(params)"),call.=FALSE)
+            
             ntimes <- length(time(object))
             if (is.null(dim(params))) {
               params <- matrix(
@@ -64,6 +65,9 @@ setMethod(
 
             npars <- nrow(params)
             paramnames <- rownames(params)
+
+            if (missing(est))
+              est <- paramnames[apply(params,1,function(x)diff(range(x))>0)]
             estind <- match(est,paramnames)
             npars.est <- length(estind)
             
@@ -138,25 +142,23 @@ setMethod(
                       )
               }
 
-              for ( j in seq_len(Np) ) {
-                x.tmp <- matrix(data=x[,j],nrow=nvars,ncol=ntries)
-                rownames(x.tmp) <- statenames
-                p.tmp <- matrix(data=params[,j],nrow=npars,ncol=ntries)
-                rownames(p.tmp) <- paramnames
-
-                ## update mean of states at time nt as per L&W AGM (1) 
-                tries <- rprocess( 
-                                  object,
-                                  xstart=x.tmp,
-                                  times=times[c(nt,nt+1)],
-                                  params=p.tmp
-                                  )
-
-                mu[,j,1] <- apply(tries[,,2,drop=FALSE],1,mean)			
-
-                ## shrink parameters towards mean as per Liu & West eq (3.3) and L&W AGM (1)
-                m[,j] <- shrink*params[,j] + (1-shrink)*params.mean 
-              }
+              X <- matrix(data=x,nrow=nvars,ncol=Np*ntries)
+              rownames(X) <- statenames
+              P <- matrix(data=params,nrow=npars,ncol=Np*ntries)
+              rownames(P) <- paramnames
+              ## update mean of states at time nt as per L&W AGM (1) 
+              tries <- rprocess(
+                                object,
+                                xstart=X,
+                                times=times[c(nt,nt+1)],
+                                params=P
+                                )[,,2,drop=FALSE]
+              dim(tries) <- c(nvars,Np,ntries,1)
+              mu <- apply(tries,c(1,2,4),mean)
+              rownames(mu) <- statenames
+              ## shrink parameters towards mean as per Liu & West eq (3.3) and L&W AGM (1)
+              m <- shrink*params+(1-shrink)*params.mean
+              
               ## evaluate probability of obervation given mean value of parameters and states (used in L&W AGM (5) below)
               g <- dmeasure( 
                             object,
@@ -166,9 +168,11 @@ setMethod(
                             params=m											
                             )	
               ## sample indices -- From L&W AGM (2)
+##              k <- .Call(systematic_resampling,g)
               k <- sample.int(n=Np,size=Np,replace=TRUE,prob=g)
               params <- params[,k]
               m <- m[,k]
+              g <- g[k]
 
               ## sample new parameter vector as per L&W AGM (3) and Liu & West eq(3.2)
               pvec <- try(
@@ -189,7 +193,7 @@ setMethod(
               ## sample current state vector x^(g)_(t+1) as per L&W AGM (4)
               X <- rprocess(
                             object,
-                            xstart=x[,k],
+                            xstart=x[,k,drop=FALSE],
                             times=times[c(nt,nt+1)],
                             params=params
                             )[,,2,drop=FALSE]
@@ -203,7 +207,7 @@ setMethod(
                                 params=params
                                 )
               ## evaluate weights as per L&W AGM (5)
-              weights <- numer/g[k]
+              weights <- numer/g
               
               ## apply box constraints as per the priors          
               for (j in seq_len(Np)) {
@@ -249,11 +253,11 @@ setMethod(
                 eff.sample.size[nt] <- 0
               } else {                  # not all particles are lost
                 ## compute log-likelihood
-                loglik[nt] <- log(mean(weights))  
+                loglik[nt] <- log(mean(weights))
                 weights[failures] <- 0
                 weights <- weights/sum(weights)
                 ## compute effective sample-size
-                eff.sample.size[nt] <- 1/(weights%*%weights) 
+                eff.sample.size[nt] <- 1/crossprod(weights)
               }
 
               if (verbose) {
