@@ -22,6 +22,10 @@ struct lookup_table {
   double *y;
 };
 
+// routine to compute number of discrete-time steps to take.
+// used by plugins in 'euler.c' and map iterator in 'trajectory.c'
+int num_map_steps (double t1, double t2, double dt);
+
 // simple linear interpolation of the lookup table (with derivative if desired)
 // setting dydt = 0 in the call to 'table_lookup' will bypass computation of the derivative
 void table_lookup (struct lookup_table *tab, double x, double *y, double *dydt);
